/* Copyright (C) 2005 David Decotigny
   Copyright (C) 2003 Thomas Petazzoni

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA. 
*/
#ifndef _SOS_USER_LIBC_H_
#define _SOS_USER_LIBC_H_

#include <types.h>

/**
 * @file libc.h
 *
 * The basic user C library for user programs
 */

/**
 * The most important function of a C program ! ;)
 */
void exit (int status) __attribute__((noreturn, alias("_sos_exit")));


/**
 * Function to duplicate the current running process
 */
pid_t fork(void)  __attribute__ ((alias("_sos_fork")));


/**
 * Function to re-initialize the address space of the current process
 * with that of the program 'progname'
 */
int exec(const char *progname) __attribute__ ((alias("_sos_exec")));


/**
 * These flags (for mmap API) MUST be consistent with those defined in
 * paging.h and umem_vmm.h
 */
#define PROT_NONE  0
#define PROT_READ  (1<<0)
#define PROT_WRITE (1<<1)
#define PROT_EXEC  (1<<2) /* Not supported on IA32 */

#define MAP_SHARED (1 << 0)
#define MAP_FIXED  (1 << 31)

/**
 * Non standard version of mmap. Differences with standard version:
 *  - the resource to map is given by its "path", not by a file descriptor
 *  - the offset is a signed 64bits, and MUST be >= 0 (error otherwise)
 *  - no errno support
 */
void * mmap(void *start, size_t length, int prot , int flags,
	    const char *path, loff_t offset);


/**
 * Unmap the given user address interval
 */
int munmap(void * start, size_t length) __attribute__ ((alias("_sos_munmap")));


/**
 * Change the access permissions of the given user address interval
 */
int mprotect(const void *addr, size_t len, int prot) __attribute__ ((alias("_sos_mprotect")));



/**
 * This flag tells mremap that the underlying VR may be moved, when necessary
 */
#define MREMAP_MAYMOVE (1 << 30)

/**
 * Grow/shrink the end of the given mapping
 */
void * mremap(void * old_addr, size_t old_len,
	      size_t new_len,
	      unsigned long flags);


/**
 * Standard heap management API
 */

/** @note MT UNSAFE */
int brk(void *end_data_seg);

/** @note MT UNSAFE */
void *sbrk(ptrdiff_t increment);

/** @note MT UNSAFE */
void * calloc (size_t nmemb, size_t size);

/** @note MT UNSAFE */
void * malloc (size_t size);


#endif /* _SOS_USER_LIBC_H_ */
